@@ -7,6 +7,7 @@ from contextlib import contextmanager
 
 from typing import Any
 from typing import Callable
+from typing import Generator
 from typing import Dict
 from typing import List
 from typing import Optional
@@ -16,8 +17,9 @@ from typing import Union
 
 from enum import Enum
 
-from pydantic import parse_obj_as
+from pydantic import TypeAdapter
 from pydantic import ValidationError
+from pydantic_core import ErrorDetails
 
 from slxjsonrpc.schema.jsonrpc import RpcBatch
 from slxjsonrpc.schema.jsonrpc import RpcError
@@ -26,12 +28,13 @@ from slxjsonrpc.schema.jsonrpc import RpcRequest
 from slxjsonrpc.schema.jsonrpc import RpcResponse
 
 from slxjsonrpc.schema.jsonrpc import ErrorModel
+from slxjsonrpc.schema.jsonrpc import MethodError
+from slxjsonrpc.schema.jsonrpc import rpc_set_name
 from slxjsonrpc.schema.jsonrpc import RpcErrorCode
 from slxjsonrpc.schema.jsonrpc import RpcErrorMsg
-from slxjsonrpc.schema.jsonrpc import rpc_set_name
+from slxjsonrpc.schema.jsonrpc import set_id_mapping
 from slxjsonrpc.schema.jsonrpc import set_params_map
 from slxjsonrpc.schema.jsonrpc import set_result_map
-from slxjsonrpc.schema.jsonrpc import set_id_mapping
 
 RpcSchemas = Union[
     RpcError,
@@ -66,9 +69,9 @@ class RpcErrorException(Exception):
             data: (Optional) The Rpc Extended error message.
         """
         super().__init__()
-        self.code = code
-        self.msg = msg
-        self.data = data
+        self.code: Union[int, RpcErrorCode] = code
+        self.msg: str = msg
+        self.data: Optional[Any] = data
 
     def get_rpc_model(self, id: Union[str, int, None]) -> RpcError:
         """
@@ -123,7 +126,7 @@ class SlxJsonRpc:
                     If not given, will there not be make checks for any wrong methods.
             method_cb: The mapping for each given method to a function call. (Server only)
                         callback: The function to be call when data is received.
-                                  The Callback gets the params definded in as args,
+                                  The Callback gets the params defined in as args,
                                   & should return the Result defined.
                                   If an error happens, and custom error code
                                   are needed, to send back, raise the RpcErrorException
@@ -133,7 +136,7 @@ class SlxJsonRpc:
             params: (Optional) The Parser method & 'params' mapping.
                     If not given, will there not be make checks for any wrong 'params'.
         """
-        self.log = logging.getLogger(__name__)
+        self.log: logging.Logger = logging.getLogger(__name__)
         self.log.addHandler(logging.NullHandler())
 
         rpc_set_name(None)
@@ -148,6 +151,8 @@ class SlxJsonRpc:
         self.__batch_lock: int = 0
         self.__batched_list: List[RpcSchemas] = []
 
+        self.__parse_as_rpc_obj: TypeAdapter = TypeAdapter(RpcSchemas)  # type: ignore
+
         self._method_cb: Dict[Union[Enum, str], Callable[[Any], Any]] = method_cb if method_cb else {}
 
         self._id_cb: Dict[Union[str, int, None], Callable[[Any], None]] = {}
@@ -175,7 +180,7 @@ class SlxJsonRpc:
             method: Should be a apart of the given Method Enum, given on init,
                     or if not given, a string.
             callback: The function to be called when data is received.
-                      The Callback gets the Result datamodel (if set)
+                      The Callback gets the Result data model (if set)
                       else a Dict/List back as argument.
             error_callback: (Optional) The function to be called, when an error
                             have happened.
@@ -194,13 +199,28 @@ class SlxJsonRpc:
             params=params
         )
 
-        self._id_cb[r_data.id] = callback
-        if error_callback:
-            self._id_error_cb[r_data.id] = error_callback
-        self._id_method[r_data.id] = method
+        self._add_result_handling(
+            method=method,
+            _id=r_data.id,
+            callback=callback,
+            error_callback=error_callback
+        )
 
         return self._batch_filter(r_data)
 
+    def _add_result_handling(
+        self,
+        method: Union[Enum, str],
+        _id: Union[str, int, None],
+        callback: Callable[[Any], None],
+        error_callback: Optional[Callable[[ErrorModel], None]] = None,
+    ) -> None:
+        """Added handling for when a result comes back."""
+        self._id_cb[_id] = callback
+        if error_callback:
+            self._id_error_cb[_id] = error_callback
+        self._id_method[_id] = method
+
     def create_notification(
         self,
         method: Union[Enum, str],
@@ -237,7 +257,7 @@ class SlxJsonRpc:
     # -------------------------------------------------------------------------
 
     @contextmanager
-    def batch(self):
+    def batch(self) -> Generator[None, None, None]:
         """Batch RPC's within the context manager, into one RPC-Batch-List."""
         self.__batch_lock += 1
         try:
@@ -271,11 +291,11 @@ class SlxJsonRpc:
             return None
         if data:
             self.__batched_list.append(data)
-        sdata = self.__batched_list.copy()
+        batched_data = self.__batched_list.copy()
         self.__batched_list.clear()
-        if len(sdata) == 1:
-            return sdata[0]
-        return parse_obj_as(RpcBatch, sdata)
+        if len(batched_data) == 1:
+            return batched_data[0]
+        return RpcBatch.model_validate(batched_data)
 
     @overload
     def _batch_filter(self, data: RpcError) -> Optional[RpcError]: ...  # noqa: E704
@@ -315,7 +335,7 @@ class SlxJsonRpc:
 
     def parser(
         self,
-        data: Union[bytes, str, dict, list]
+        data: Union[bytes, str, Dict[str, Any], List[Dict[str, Any]]]
     ) -> Optional[Union[RpcError, RpcResponse, RpcBatch]]:
         """
         Parse raw JsonRpc data, & returns the Response or Error.
@@ -336,7 +356,7 @@ class SlxJsonRpc:
             None, if no reply are needed.
         """
         try:
-            j_data: Union[dict, list]
+            j_data: Union[Dict[str, Any], List[Dict[str, Any]]]
             if isinstance(data, dict) or isinstance(data, list):
                 j_data = data
             else:
@@ -375,9 +395,9 @@ class SlxJsonRpc:
                     r_data = self.__reply_logic(temp)
                     if r_data:
                         b_data.append(r_data)
-                # UNSURE: Is it requerid to return a batch of 1, if it was received as batch of 1?
+                # UNSURE: Is it required to return a batch of 1, if it was received as batch of 1?
 
-            return parse_obj_as(RpcBatch, b_data) if b_data else None
+            return RpcBatch.model_validate(b_data) if b_data else None
 
         try:
             p_data = self._parse_data(j_data)
@@ -390,7 +410,7 @@ class SlxJsonRpc:
     def __reply_logic(
         self,
         p_data: RpcSchemas
-    ) -> Union[RpcError, RpcResponse, None]:
+    ) -> Optional[Union[RpcResponse, RpcError]]:
         try:
             if isinstance(p_data, RpcError):
                 return self._error_reply_logic(data=p_data)
@@ -422,7 +442,7 @@ class SlxJsonRpc:
 
         return None
 
-    def _error_reply_logic(self, data):
+    def _error_reply_logic(self, data: RpcError) -> Optional[RpcError]:
         if data.id not in self._id_cb.keys():
             # NOTE: Triggers only if it was an error that we generated.
             return data
@@ -434,8 +454,9 @@ class SlxJsonRpc:
                 cb = self._id_error_cb.pop(data.id)
                 self.log.debug(f"Exec Error CB: {cb}")
                 cb(data.error)
+        return None
 
-    def _notification_reply_logic(self, data):
+    def _notification_reply_logic(self, data: RpcNotification) -> Optional[RpcError]:
         if data.method not in self._method_cb.keys():
             return self._batch_filter(RpcError(
                 id=None,
@@ -449,8 +470,9 @@ class SlxJsonRpc:
             cb = self._method_cb[data.method]
             self.log.debug(f"Exec Notification CB: {cb}")
             cb(data.params)
+        return None
 
-    def _request_reply_logic(self, data):
+    def _request_reply_logic(self, data: RpcRequest) -> Optional[Union[RpcResponse, RpcError]]:
         if data.method in self._method_cb.keys():
             with self._except_handler():
                 cb = self._method_cb[data.method]
@@ -469,7 +491,7 @@ class SlxJsonRpc:
             )
         ))
 
-    def _response_reply_logic(self, data):
+    def _response_reply_logic(self, data: RpcResponse) -> Optional[RpcResponse]:
         if data.id not in self._id_cb.keys():
             self.log.warning(f"Received an unknown RpcResponse: {data}")
         else:
@@ -478,15 +500,17 @@ class SlxJsonRpc:
                 cb = self._id_cb.pop(data.id)
                 self.log.debug(f"Exec Response CB: {cb}")
                 cb(data.result)
+        return None
 
     @contextmanager
-    def _except_handler(self):
+    def _except_handler(self) -> Generator[None, None, None]:
         try:
             yield
         except RpcErrorException:
             raise
         except Exception as err:
-            # NOTE: Only trickered from user given function, or if it was not a function
+            # NOTE: Only triggered from user given function, or if it was not a function
+            self.log.exception("An error happened doing execution of a callback.")
             raise RpcErrorException(
                 code=RpcErrorCode.ServerError,
                 msg=RpcErrorMsg.ServerError,
@@ -495,19 +519,12 @@ class SlxJsonRpc:
 
     def __ValidationError2ErrorModel(
         self,
-        errors: List[Dict[str, Union[List[str], str, Dict[str, List[str]]]]]
+        errors: List[ErrorDetails]
     ) -> Optional[ErrorModel]:
         # TODO (MBK): Find a faster/better way to do this!
-        method_error = list(filter(lambda x: x.get('type') == "type_error.enum", errors))
-        params_error = list(filter(lambda x: x.get('loc') == ('__root__', 'params', '__root__'), errors))
-        type_error = list(filter(lambda x: x.get('type') in ["value_error.missing", "value_error.extra"], errors))
-        if method_error:
-            raise RpcErrorException(
-                code=RpcErrorCode.MethodNotFound,
-                msg=RpcErrorMsg.MethodNotFound,
-                data=method_error[0]
-            )
-        elif params_error:
+        params_error = list(filter(lambda x: x['loc'][1] == 'params', errors))
+        type_error = list(filter(lambda x: x['type'] in ["missing", "extra_forbidden"], errors))
+        if params_error:
             raise RpcErrorException(
                 code=RpcErrorCode.InvalidParams,
                 msg=RpcErrorMsg.InvalidParams,
@@ -524,12 +541,15 @@ class SlxJsonRpc:
 
     def _parse_data(
         self,
-        data: dict
+        data: Dict[str, Any]
     ) -> RpcSchemas:
         try:
-            p_data: RpcSchemas = parse_obj_as(
-                RpcSchemas,  # type: ignore
-                data
+            p_data: RpcSchemas = self.__parse_as_rpc_obj.validate_python(data)
+        except MethodError as error:
+            raise RpcErrorException(
+                code=RpcErrorCode.MethodNotFound,
+                msg=RpcErrorMsg.MethodNotFound,
+                data=error.args[0]
             )
         except ValidationError as error:
             error_package = self.__ValidationError2ErrorModel(
